/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.uuid.framework.orchestrator;

import java.util.Set;
import java.util.TreeSet;

import com.ebmwebsourcing.easycommons.thread.SimultaneousTaskExecutor;

public abstract class AbstractOrchestrator {

    private SimultaneousTaskExecutor simultaneousThreadExecutor = new SimultaneousTaskExecutor();

    protected abstract Runnable getGeneratorTask(final int nbIterates, final Set<String> uuids/*,
            final TestState testState*/);

    public class TestState {
        private boolean success = true;

        public boolean isSuccess() {
            return success;
        }

        public void setSuccess(boolean success) {
            this.success = success;
        }
    }

    public final void orchestrate(final int poolSize, final int iterations) throws Exception {

        // A set of generated uuids, used to test if no doublon is generated
        final Set<String> uuids = new TreeSet<String>();

        for (int i = 0; i < poolSize; i++) {
            Runnable task = this.getGeneratorTask(iterations, uuids);
            simultaneousThreadExecutor.registerTask(task);
        }
        
        simultaneousThreadExecutor.executeAllRegisteredTasks();
        
        System.out.println(uuids.size());
    }

}
